% To use the code, please refer to our paper 
%"Bottom-up Saliency based on Weighted Sparse Coding Residual" ACM MM 2011
% By Biao Han, SHU, raintwoto@gmail.com, Hao Zhu, BNU, zhuhao_hit@hotmail.com

% The code can be downloaded from my website: http://biao.compneurosci.net/

% Some of the codes are from 
%"Incremental Sparse Saliency Detection" ICIP 2009 By Yin Li, SJTU

% alf=0.1;
%for efficiency, we only consider three scales and generate the master_map 
% m and n are the width or heights of the original image
% size_v = [ceil(n.*alf) ceil(m.*alf)];
% size_h = [ceil(m.*alf) ceil(n.*alf)];

%The image is decomposed into sereval patches, each patch is encoded by the
%surrounding patches.

% SPAMS toolbox is required for the code. http://spams-devel.gforge.inria.fr/

%INPUT: 
%OriginalImage   -- the input gray image
%blockSize -- size of the patch, default: 8
%alpha -- size of the neighborhood (MUST be odd: 5, 7, 9...), default: 7 
%lambda -- param for saliency, default: 0.01


%OUTPUT:
%map -- final saliency map


function [map] = ACMMM11(org_img, blockSize, alpha, lambda)
%%
%init
[m n c] = size(org_img);
alf=0.1;
size_h = [ceil(m.*alf) ceil(n.*alf)];
img = imresize(org_img,size_h, 'bicubic');

%parameter for the Lasso
param.lambda = lambda;
param.mode = 2;

%pre allocated memory
map = zeros([m n]);
center = zeros([blockSize*blockSize*c 1]);
tempPatch = zeros([blockSize blockSize c]);
stepLength = fix(blockSize./2);                %half overlapping
csLength = fix(blockSize./2);
total = round((m-blockSize)./stepLength)*round((n-blockSize)./stepLength);

%%
%scan for center-surround archetecture
fprintf(' Weighted Sparse Coding Residual: Calculating block saliency.\n');
progress = 0;
fprintf('0        25       50       75        100\n');                                                 
tic
for i=1:stepLength:m-blockSize+1
    for j=1:stepLength:n-blockSize+1
        
        %clear the memory
        A = zeros([blockSize*blockSize*c 1]);
        
        %location of the center patch
        ctop = i;
        cbottom = i+blockSize-1;  
        cleft = j;
        cright = j+blockSize-1;
               
        %location of the surroundings patch
        stop = max([1 i-fix(alpha./2)*blockSize]);
        sbottom = min([m i+blockSize-1+fix(alpha./2)*blockSize]); 
        sleft = max([1 j-fix(alpha./2)*blockSize]);
        sright = min([n j+blockSize-1+fix(alpha./2)*blockSize]);
        sm = sbottom-stop+1;
        sn = sright-sleft+1;
        
        %location of the center in the surrounding patch
        top = ctop-stop+1;
        bottom = top+blockSize-1;
        left = cleft-sleft+1;
        right = left+blockSize-1;
        
        %vector of the center
        tempPatch = img(ctop:cbottom,cleft:cright,:);
        center = tempPatch(1:blockSize*blockSize*c)';
               
        %get all the surrounding patches to form overcomplete dictionary A
        count=1;
        for k=1:csLength:sm-blockSize+1
            for l=1:csLength:sn-blockSize+1
                if k+blockSize-1<top || k>bottom || l+blockSize-1<left || l>right
                    tempPatch = img(stop+k-1:stop+k-2+blockSize,sleft+l-1:sleft+l-2+blockSize,:);
                    A(:,count) = tempPatch(1:blockSize*blockSize*c)';
                    count = count+1;
                end
            end
        end
        
        % sparse coding           
        x = mexLasso(center, A, param);
        
        % weighted sparse coding residual          
        map(ctop:cbottom,cleft:cright) = map(ctop:cbottom,cleft:cright)+length(x)*sum(abs(x)>0)*sum(abs(center-A*x));
             
        %for display
        progress = progress+1./total; 
        if (progress > 0.025)
            progress = progress - 0.025;
            fprintf('.')
        end
        
    end
end
toc

%%
%normalize and blur 
g = fspecial('gaussian',[8 8],8);
map = imfilter(map,g).^2;
map = (map-min(min(map)))./(max(max(map))-min(min(map)));
map= imresize(map,[m,n]);